/*
 * Copyright (c) 2003, Morpho Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the Morpho Project nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.morpho.webapp.servlet.jsp.tagext;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderMap;
import jp.morpho.webapp.FrameworkContext;
import jp.morpho.webapp.FrameworkException;
import jp.morpho.webapp.PluginKeys;
import jp.morpho.webapp.action.ActionMappingManager;
import jp.morpho.webapp.servlet.jsp.tagext.TagManager;

/**
 * <P>
 * åܥå饸ܥ󡢥˥塼ʤɤΥȥϥɥ뤿δ쥯饹Ǥ
 * </P>
 * @author Kumiko Hiroi
 */
public abstract class BaseTag
	extends BodyTagSupport
{
	private ConversionMap attrs = new ConversionMap();
	private boolean bodyExists = false;

	/**
	 * <P>
	 * ǥեȤΥ󥹥ȥ饯Ǥ
	 * ٤ƤΥ֥饹ϡƱ˥㡼 public 󥹥ȥ饯ѡ饹Υ󥹥ȥ饯ƤӽФɬפޤ
	 * Υ󥹥ȥ饯ϡJSP ȥ󥹥졼ˤä줿ɤƤӽФޤ
	 * </P>
	 */
	public BaseTag()
	{
		super();
	}

	/**
	 * <P>
	 * Ǥ° jp.morpho.util.PrimitiveHolderMap ֤ޤ
	 * </P>
	 * @return Ȥ°̾ޥåͤȤ°ͤǼ줿 jp.morpho.util.PrimitiveHolderMap°ޥåΥͤ String 
	 */
	public PrimitiveHolderMap getAttributeMap()
	{
		return this.attrs;
	}

	/**
	 * <P>
	 * λνǤ
	 * </P>
	 * @return EVAL_PAGE
	 * @see javax.servlet.jsp.tagext.Tag#doEndTag()
	 */
	public int doEndTag() throws JspException
	{
		this.getAttributeMap().clear();
		this.bodyExists = false;
		return super.doEndTag();
	}

	/**
	 * <P>
	 * ƤӽФ߼¹ԤƤ ServletContext ˳ǼƤ FrameworkContext ֥ȤؤλȤޤ
	 * </P>
	 * @return ƤӽФ֥åȥƥʤȤäΤ˻ѤƤ ServletContext ֥Ȥ˳ǼƤ FrameworkContext ֥
	 */
	protected FrameworkContext getFrameworkContext() throws JspException
	{
		FrameworkContext fc;
		try
		{
			fc = FrameworkContext.getInstance(((HttpServletRequest)pageContext.getRequest()).getSession().getServletContext());
		}
		catch (FrameworkException e)
		{
			throw new JspException(e.getMessage());
		}
		return fc;
	}

	/**
	 * <P>
	 * ƤӽФ߼¹ԤƤ ServletContext ˳ǼƤ FrameworkContext ֥Ȥ顢TagManager ޤ
	 * </P>
	 * @return TagManager ֥
	 */
	protected TagManager getTagManager() throws JspException
	{
		FrameworkContext fc = this.getFrameworkContext();
		TagManager tm = (TagManager)fc.get(PluginKeys.fieldContext);
		if (tm == null)
		{
			throw new JspException("TagManager is not found.");
		}
		return tm;
	}

	/**
	 * <P>
	 * ƤӽФ߼¹ԤƤ ServletContext ˳ǼƤ FrameworkContext ֥Ȥ顢ActionMappingManager ޤ
	 * </P>
	 * @return ActionMappingManager ֥
	 */
	protected ActionMappingManager getActionMappingManager() throws JspException
	{
		FrameworkContext fc = this.getFrameworkContext();
		ActionMappingManager amm = (ActionMappingManager)fc.get(PluginKeys.actionMappingManager);
		if (amm == null)
		{
			throw new JspException("ActionMappingManager is not found.");
		}
		return amm;
	}

	/**
	 * <P>
	 * ΤɾˡɾԤ鷺ˡڡ³Ԥޤ
	 * ǥեȤǤϡbodyContent Υǡ (¸ߤ) ޤ
	 * </P>
	 * @return SKIP_BODY
	 * @see javax.servlet.jsp.tagext.IterationTag#doAfterBody()
	 */
	public int doAfterBody() throws JspException
	{
		bodyExists = true;
		if (bodyContent.getString().length() > 0)
		{
			try
			{
				bodyContent.writeOut(bodyContent.getEnclosingWriter());
			}
			catch (IOException e)
			{
				throw new JspException(e.getMessage());
			}
		}
		return super.doAfterBody();
	}

	/**
	 * <P>
	 * Τ true ֤ޤ
	 * </P>
	 * @return Τ true
	 */
	public boolean isBodyExists()
	{
		return bodyExists;
	}

}
